/**
 * Copyright (c) 2015 Digi International Inc.,
 * All rights not expressly granted are reserved.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Digi International Inc. 11001 Bren Road East, Minnetonka, MN 55343
 * =======================================================================
 */

#include "mbed.h"
#include "XBeeLib.h"
#if defined(ENABLE_LOGGING)
#include "DigiLoggerMbedSerial.h"
using namespace DigiLog;
#endif

using namespace XBeeLib;

/* Configure Sleep Options */
#define SLEEP_OPTIONS 0      /* Short Sleep */

/* Configure Sleep Period in mS */
#define SLEEP_PERIOD_MS (5000 / 10)   /* 5000 mS */

/* Configure Time before sleep */
#define TIME_BEFORE_SLEEP_MS 500    /* 500 mS */

/* Configure Number of sleep periods */
#define SLEEP_PERIOD_NUMBER 65535    /* High value so radio doesn't awake unnecessarily */

#define REMOTE_NODE_ADDR64_MSB  ((uint32_t)0x0013A200)

#error "Replace next define with the LSB of the remote module's 64-bit address (SL parameter)"
#define REMOTE_NODE_ADDR64_LSB  ((uint32_t)0x01234567)

#define REMOTE_NODE_ADDR64      UINT64(REMOTE_NODE_ADDR64_MSB, REMOTE_NODE_ADDR64_LSB)

Serial *log_serial;

/** Callback function, invoked at packet reception */
static void receive_cb(const RemoteXBeeDM& remote, bool broadcast, const uint8_t *const data, uint16_t len)
{
    const uint64_t remote_addr64 = remote.get_addr64();

    log_serial->printf("\r\nGot a %s RX packet [%08x:%08x|%04x], len %d\r\nData: ", broadcast ? "BROADCAST" : "UNICAST",
                                      UINT64_HI32(remote_addr64), UINT64_LO32(remote_addr64), remote.get_addr16(), len);

    log_serial->printf("%.*s\r\n\r\n", len, data);
}

int main()
{
    log_serial = new Serial(DEBUG_TX, DEBUG_RX);
    log_serial->baud(9600);
    log_serial->printf("Sample application to demo cyclic sleep power management with the XBeeDM\r\n\r\n");
    log_serial->printf(XB_LIB_BANNER);

#if defined(ENABLE_LOGGING)
    new DigiLoggerMbedSerial(log_serial, LogLevelInfo);
#endif

    XBeeDM xbee = XBeeDM(RADIO_TX, RADIO_RX, RADIO_RESET, NC, NC, 9600);

    const RemoteXBeeDM remoteDevice = RemoteXBeeDM(REMOTE_NODE_ADDR64);

    /* Register callbacks */
    xbee.register_receive_cb(&receive_cb);

    RadioStatus radioStatus = xbee.init();
    MBED_ASSERT(radioStatus == Success);

    /* Configure sleep parameters */
    AtCmdFrame::AtCmdResp cmdresp;

    cmdresp = xbee.set_param("SO", (uint32_t)SLEEP_OPTIONS);
    if (cmdresp != AtCmdFrame::AtCmdRespOk) {
        log_serial->printf("Setting SO Failed!!\r\n");
        goto end;
    }

    cmdresp = xbee.set_param("SP", SLEEP_PERIOD_MS);
    if (cmdresp != AtCmdFrame::AtCmdRespOk) {
        log_serial->printf("Setting SP Failed!!\r\n");
        goto end;
    }

    cmdresp = xbee.set_param("ST", TIME_BEFORE_SLEEP_MS);
    if (cmdresp != AtCmdFrame::AtCmdRespOk) {
        log_serial->printf("Setting ST Failed!!\r\n");
        goto end;
    }

    cmdresp = xbee.set_param("SN", SLEEP_PERIOD_NUMBER);
    if (cmdresp != AtCmdFrame::AtCmdRespOk) {
        log_serial->printf("Setting SN Failed!!\r\n");
        goto end;
    }

    cmdresp = xbee.set_param("CE", 4);  /* Indirect Message Poller */
    if (cmdresp != AtCmdFrame::AtCmdRespOk) {
        log_serial->printf("Setting CE Failed!!\r\n");
        log_serial->printf("This example only works if both devices are S2C modules where asynchronous cyclic sleep is supported!!\r\n");
        goto end;
    }

    /* Configure DH/DL with the address of the remoter device that acts 
       as indirect message coordinator. Poll messages will be sent to this 
       address each time the device awakes to check for pending messages */
    radioStatus = xbee.config_poll_destination(remoteDevice);
    if (radioStatus != Success) {
        log_serial->printf("config_poll_destination Failed!!\r\n");
        goto end;     
    }

    /* Configure Sleep mode */
    cmdresp = xbee.set_param("SM", 4);  /* Cyclic Sleep */
    if (cmdresp != AtCmdFrame::AtCmdRespOk) {
        log_serial->printf("Setting SM Failed!!\r\n");
        goto end;
    }

    /* Start processing frames */
    while (true) {
        xbee.process_rx_frames();
        wait_ms(100);
        log_serial->printf(".");
    }

end:
    delete(log_serial);
}
